import unittest2 as unittest
import doctest
from plone.testing import layered
from plone.multilingual.testing import (
    PLONEMULTILINGUAL_INTEGRATION_TESTING,
    PLONEMULTILINGUAL_FUNCTIONAL_TESTING,
    optionflags,
)

integration_tests = [
    'README.rst',
]
functional_tests = [
]


def test_suite():
    return unittest.TestSuite(
        [layered(doctest.DocFileSuite('%s' % f, package='plone.multilingual',
                                      optionflags=optionflags),
                 layer=PLONEMULTILINGUAL_INTEGRATION_TESTING)
            for f in integration_tests]
        +
        [layered(doctest.DocFileSuite('%s' % f, package='plone.multilingual',
                                      optionflags=optionflags),
                 layer=PLONEMULTILINGUAL_FUNCTIONAL_TESTING)
            for f in functional_tests]
        )

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



# -*- coding: utf-8 -*-
from plone.multilingual.interfaces import (
    ITranslationFactory,
    ILanguageIndependentFieldsManager,
    ITranslationLocator,
    ITranslationCloner,
    ITranslationIdChooser,
    ITranslationManager,
)
from zope import interface
from Acquisition import aq_parent
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot


class DefaultLanguageIndependentFieldsManager(object):
    """ Default language independent fields manager.
    """
    interface.implements(ILanguageIndependentFieldsManager)

    def __init__(self, context):
        self.context = context

    def get_field_names(self):
        return []

    def copy_fields(self, translation):
        return


class DefaultTranslationLocator(object):

    interface.implements(ITranslationLocator)

    def __init__(self, context):
        self.context = context

    def __call__(self, language):
        parent = aq_parent(self.context)
        if not IPloneSiteRoot.providedBy(parent):
            parent_translation = ITranslationManager(parent)
            if parent_translation.has_translation(language):
                parent = parent_translation.get_translation(language)
        return parent


class DefaultTranslationCloner(object):

    interface.implements(ITranslationCloner)

    def __init__(self, context):
        self.context = context

    def __call__(self, object):
        return


class DefaultTranslationIdChooser(object):

    interface.implements(ITranslationIdChooser)

    def __init__(self, context):
        self.context = context

    def __call__(self, parent, language):
        content_id = self.context.getId()
        splitted = content_id.split('-')
        # ugly heuristic (searching for something like 'de', 'en' etc.)
        if len(splitted) > 1 and len(splitted[-1]) == 2:
            content_id = ''.join(splitted[:-1])
        while content_id in parent.objectIds():
            content_id = "%s-%s" % (content_id, language)
        return content_id


class DefaultTranslationFactory(object):

    interface.implements(ITranslationFactory)

    def __init__(self, context):
        self.context = context

    def __call__(self, language):
        content_type = self.context.portal_type
        # parent for translation
        locator = ITranslationLocator(self.context)
        parent = locator(language)
        # id for translation
        name_chooser = ITranslationIdChooser(self.context)
        content_id = name_chooser(parent, language)
        # creating the translation
        new_id = parent.invokeFactory(type_name=content_type, \
            id=content_id, language=language)
        new_content = getattr(parent, new_id)
        # clone language-independent content
        cloner = ITranslationCloner(self.context)
        cloner(new_content)
        return new_content



# BlueDynamics Alliance
# http://www.bluedynamics.com.com

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

__author__ = """Jens Klein <jens@bluedynamics.com>"""
__docformat__ = 'plaintext'

from zope.interface import implements
from zope.interface import Attribute
from zope.component.interfaces import IObjectEvent


class IObjectWillBeTranslatedEvent(IObjectEvent):
    """Sent before an object is translated."""

    object = Attribute("The object to be translated.")
    language = Attribute("Target language.")


class IObjectTranslatedEvent(IObjectEvent):
    """Sent after an object was translated."""

    object = Attribute("The object to be translated.")
    target = Attribute("The translation target object.")
    language = Attribute("Target language.")


class ObjectWillBeTranslatedEvent(object):
    """Sent before an object is translated."""
    implements(IObjectWillBeTranslatedEvent)

    def __init__(self, context, language):
        self.object = context
        self.language = language


class ObjectTranslatedEvent(object):
    """Sent after an object was translated."""
    implements(IObjectTranslatedEvent)

    def __init__(self, context, target, language):
        self.object = context
        self.target = target
        self.language = language



# -*- coding: utf-8 -*-
from zope.interface import implements
from BTrees.OOBTree import OOBTree
from OFS.SimpleItem import SimpleItem
from plone.multilingual.interfaces import IMultilingualStorage


class CanonicalStorage(SimpleItem):

    implements(IMultilingualStorage)
    id = 'portal_multilingual'

    def __init__(self):
        self.id = id
        self.canonicals = OOBTree()

    def get_canonical(self, id):
        """ get a canonical for a specific content-id """
        canonical = None
        if id in self.canonicals:
            canonical = self.canonicals[id]
        return canonical

    def add_canonical(self, id, canonical):
        """ add a canonical """
        self.canonicals.insert(id, canonical)

    def remove_canonical(self, id):
        """ remove a canonical """
        self.canonicals.pop(id)
